import pymatgen
import general_AF_structure_analysis as g
from pymatgen.core import Structure
from pymatgen.core import Lattice
import copy
import numpy as np 
from numpy import sqrt
from sympy import Point,Triangle
import pickle as pkl
from pymatgen.io.vasp import Poscar
import glob

BN_radii_dict= {'ClClCl': [1.81, 1.81, 1.81], 'ClClN':[1.46,1.81,1.81], 'ClNN': [1.46,1.46,1.84], 'NNN':[1.46, 1.46, 1.46]}
BN_radii_dict_tet= {'ClCl':[1.81,1.81], 'ClN': [1.46,1.81], 'NN':[1.46, 1.46]}
BN_size_dictionary = {**{k: [] for k,v in BN_radii_dict.items()},
  **{k+'_modified': [] for k,v in BN_radii_dict.items()},
**{k+'_average': 0 for k,v in BN_radii_dict.items()},
  **{k+'_modified_average': 0 for k,v in BN_radii_dict.items()},
**{k+'_error': 0 for k,v in BN_radii_dict.items()},
  **{k+'_modified_error': 0 for k,v in BN_radii_dict.items()}, 
  #for tet
  **{k: [] for k,v in BN_radii_dict_tet.items()},
  **{k+'_modified': [] for k,v in BN_radii_dict_tet.items()},
  **{k+'_average': 0 for k,v in BN_radii_dict_tet.items()},
  **{k+'_modified_average': 0 for k,v in BN_radii_dict_tet.items()},
  **{k+'_error': 0 for k,v in BN_radii_dict_tet.items()},
  **{k+'_modified_error': 0 for k,v in BN_radii_dict_tet.items()}}


LNS = Structure.from_spacegroup("Fm-3m", Lattice.cubic(5.578), ['Cl-', "Na+", "Li+"], [[0, 0, 0], [0, 0.5, 0], [0.25, 0.25, 0.25]])*(2,2,2)
def put_ideal_Li_sites_into_structure_and_get_polyhedra(structure):
    # structure = Structure.from_file('Supercell_1_fully_relxed_0K.vasp')
    
    #Add sodium into the structure to identify the octahedra
    
    structure_with_ideal_sites = copy.deepcopy(structure)
    structure_with_ideal_sites.remove_species(['Li'])
    
    #Becasue the first 32 sites are Cl
    for site in LNS[32:]:
        structure_with_ideal_sites.append(site.specie, site.frac_coords, coords_are_cartesian=False)
    
    structure_with_ideal_sites.sort()
    
    structure_with_polyhedra_analysed = g.find_tet_for_Li(structure_with_ideal_sites)
    return structure_with_polyhedra_analysed

files = glob.glob('./5_structures_for_bottleneck_size_analysis/*')   
structures = [Poscar.from_file(file).structure for file in files]

for s in structures:
    
    structure_with_polyhedra_analysed = put_ideal_Li_sites_into_structure_and_get_polyhedra(s)

    oct_tet_pair_dictionary = g.make_pair_dictionary_oct_tet(structure_with_polyhedra_analysed)
    tet_tet_pair_dictionary = g.make_pair_dictionary_tet_tet(structure_with_polyhedra_analysed)
    
    
    
    
    for k,v in oct_tet_pair_dictionary.items():
        
        
        a = v['shared_sites']  
        BN =  v['BN']
        radii = BN_radii_dict[BN]
        
        lengths = [np.linalg.norm(a[0].coords - a[1].coords), np.linalg.norm(a[0].coords - a[2].coords), np.linalg.norm(a[1].coords - a[2].coords)]
        
        t = Triangle(sss = lengths)
        
        p1, p2, p3 = list(t.vertices[0]), list(t.vertices[1]), list(t.vertices[2])
        p1,p2,p3= np.array([float(p1[0]), float(p1[1])]),np.array([float(p2[0]), float(p2[1])]), np.array([float(p3[0]), float(p3[1])])
        
        BN_size_dictionary[BN].append(float(t.inradius))
        # BN_size_dictionary[BN+'_average'] = np.mean(BN_size_dictionary[BN])
        # BN_size_dictionary[BN+'_error'] = np.std(BN_size_dictionary[BN])/np.sqrt(len(BN_size_dictionary[BN]))
    
        
        new_points = []
        for v, point, r in zip(t.vertices, [p1,p2,p3], radii):
            a = t.bisectors()[v]
            b = np.array(list(a.points[-1].coordinates)) - np.array(list(a.points[0].coordinates)) 
            
            b[0], b[1] = float(b[0]), float(b[1])
            # print(b)
            new_points.append(point+b*r/np.linalg.norm(b))
        t2 = Triangle(Point(new_points[0]), Point(new_points[1]), Point(new_points[2]))
        
        
        BN_size_dictionary[BN+'_modified'].append(float(t2.circumradius))
        # BN_size_dictionary[BN+'_modified_average'] = np.mean(BN_size_dictionary[BN+'_modified'])
        # BN_size_dictionary[BN+'_modified_error'] = np.std(BN_size_dictionary[BN+'_modified'])/np.sqrt(len(BN_size_dictionary[BN+'_modified']))
        
    
    #Now for tet
        
    for k,v in tet_tet_pair_dictionary.items():
        a = v['shared_sites'] 
        BN = v['BN']
        radii = BN_radii_dict_tet[BN]
        length = np.linalg.norm(a[0].coords - a[1].coords)
        length_minus_peripheral_radius = length - radii[0] - radii[1]
        BN_size_dictionary[BN].append(length)
        BN_size_dictionary[BN+'_modified'].append(length_minus_peripheral_radius)
        

#Get the averages    
for BN,v in BN_radii_dict.items():
    try:
        BN_size_dictionary[BN+'_average'] = np.mean(BN_size_dictionary[BN])
        BN_size_dictionary[BN+'_error'] = np.std(BN_size_dictionary[BN])/np.sqrt(len(BN_size_dictionary[BN]))
        BN_size_dictionary[BN+'_modified_average'] = np.mean(BN_size_dictionary[BN+'_modified'])
        BN_size_dictionary[BN+'_modified_error'] = np.std(BN_size_dictionary[BN+'_modified'])/np.sqrt(len(BN_size_dictionary[BN+'_modified']))
    except:
        pass

for BN,v in BN_radii_dict_tet.items():
    try:
        BN_size_dictionary[BN+'_average'] = np.mean(BN_size_dictionary[BN])
        BN_size_dictionary[BN+'_error'] = np.std(BN_size_dictionary[BN])/np.sqrt(len(BN_size_dictionary[BN]))
        BN_size_dictionary[BN+'_modified_average'] = np.mean(BN_size_dictionary[BN+'_modified'])
        BN_size_dictionary[BN+'_modified_error'] = np.std(BN_size_dictionary[BN+'_modified'])/np.sqrt(len(BN_size_dictionary[BN+'_modified']))
    except:
        pass
    
with open('BN_sizes_dictionary.pkl', 'wb') as h:
    pkl.dump(BN_size_dictionary, h)

